@@ -48,6 +48,7 @@ gboolean verbose;
 #endif
 static GHashTable *included_fonts;
 static gboolean debug = FALSE;
+static GString *fontmap_path = NULL;
 static GSList *afm_path = NULL;
 static GSList *pfb_path = NULL;
 static GHashTable *afm_assignments;
@@ -429,9 +430,12 @@ write_fontmap( xmlDoc *fontmap, const gchar *location )
 static void
 check_directory(void)
 {
-  const char *dirname = DATADIR "/fonts";
-
   struct stat info;
+  char *dirname = DATADIR "/fonts";
+  if (fontmap_path != NULL) 
+    {
+       dirname = fontmap_path->str;
+    }
   if ( stat( dirname, &info ) )
     {
       switch ( errno )
@@ -539,6 +543,10 @@ add_path(poptContext ctx,
 	  pfb_path = g_slist_append(pfb_path, (char *) arg);
 	}
     }
+  else if(opt->shortName == 'f')
+    {
+        fontmap_path = g_string_new((gchar *) arg);
+    }
   /* else something's weird :) */
 }
 
@@ -554,6 +562,8 @@ static const struct poptOption options[] = {
   /* We want to use a callback for this launch-plugin thing so we
      can get multiple opts */
   { NULL, '\0', POPT_ARG_CALLBACK, &add_path, 0 },
+  { "fontmap-path", 'f', POPT_ARG_STRING, NULL, 0,
+    N_("Use this path to write the fontmap file."), N_("PATH") },
   { "afm-path", 'a', POPT_ARG_STRING, NULL, 0,
     N_("Use this path to search for relative afm files."), N_("PATH") },
   { "pfb-path", 'p', POPT_ARG_STRING, NULL, 0,
@@ -603,7 +613,8 @@ gint
 main( gint argc, gchar *argv[])
 {
   xmlDoc *fontmap;
-  const gchar *fontmap_loc = DATADIR "/fonts/fontmap";
+  GString *fontmap_file = NULL;
+  gchar *fontmap_loc = DATADIR "/fonts/fontmap";
   gint i;
   gboolean scan_dot = TRUE;
   char **args;
@@ -620,6 +631,13 @@ main( gint argc, gchar *argv[])
 
   check_directory();
 
+  if (fontmap_path != NULL) 
+    {
+       fontmap_file = g_string_append(fontmap_path,"/fontmap");
+       fontmap_loc = fontmap_file->str;
+    }
+    printf("%s\n",fontmap_loc);
+
   included_fonts = g_hash_table_new( g_format_name_hash,
 				     g_format_name_equal );
 
