@@ -8,6 +8,7 @@
  * Authors: Federico Mena <federico@nuclecu.unam.mx>
  *          Raph Levien <raph@acm.org>
  *          Lauris Kaplinski <lauris@ariman.ee>
+ *          Miguel de Icaza <miguel@kernel.org>
  */
 
 /* These includes are set up for standalone compile. If/when this codebase
@@ -20,6 +21,7 @@
 #include <libart_lgpl/art_rect.h>
 #include <libart_lgpl/art_vpath.h>
 #include <libart_lgpl/art_bpath.h>
+#include <libart_lgpl/art_vpath.h>
 #include <libart_lgpl/art_vpath_bpath.h>
 #include <libart_lgpl/art_svp.h>
 #include <libart_lgpl/art_svp_vpath.h>
@@ -180,7 +182,8 @@ gnome_canvas_bpath_destroy (GtkObject *object)
 
 	if (bpath->priv->bpath)
 		art_free (bpath->priv->bpath);
-
+	g_free (bpath->priv);
+	
 	if (bpath->fill_stipple)
 		gdk_bitmap_unref (bpath->fill_stipple);
 
@@ -254,7 +257,6 @@ get_bounds_canvas (GnomeCanvasBpath *bpath, double *bx1, double *by1, double *bx
 	GnomeCanvasItem *item;
 	ArtDRect bbox_world;
 	ArtDRect bbox_canvas;
-	double i2w[6], w2c[6], i2c[6];
 
 	item = GNOME_CANVAS_ITEM (bpath);
 
@@ -374,6 +376,17 @@ set_outline_gc_width (GnomeCanvasBpath *bpath)
 				    GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
 }
 
+static gulong
+set_color_from_rgba (GnomeCanvasItem *item, GdkGC *gc, gint32 rgba_color)
+{
+	gulong pix;
+
+	pix = gnome_canvas_get_color_pixel (item->canvas, rgba_color);
+	set_gc_foreground (gc, pix);
+
+	return pix;
+}
+
 static void
 gnome_canvas_bpath_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
@@ -430,6 +443,8 @@ gnome_canvas_bpath_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	case ARG_FILL_COLOR_RGBA:
 		bp->fill_set = TRUE;
 		bp->fill_rgba = GTK_VALUE_UINT (*arg);
+		if (!item->canvas->aa)
+			bp->fill_pixel = set_color_from_rgba (item, bp->fill_gc, bp->fill_rgba);
 
 		/* should probably request repaint on the fill_svp */
 		gnome_canvas_item_request_update (item);
@@ -466,6 +481,8 @@ gnome_canvas_bpath_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	case ARG_OUTLINE_COLOR_RGBA:
 		bp->outline_set = TRUE;
 		bp->outline_rgba = GTK_VALUE_UINT (*arg);
+		if (!item->canvas->aa)
+			bp->outline_pixel = set_color_from_rgba (item, bp->outline_gc, bp->outline_rgba);
 
 		/* should probably request repaint on the outline_svp */
 		gnome_canvas_item_request_update (item);
@@ -546,9 +563,6 @@ static void
 gnome_canvas_bpath_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
 	GnomeCanvasBpath *bp;
-	GnomeCanvasBpathDef *bpath;
-	ArtBpath *art_bpath;
-	GdkColor *color;
 
 	bp = GNOME_CANVAS_BPATH (object);
 
@@ -605,7 +619,6 @@ gnome_canvas_bpath_render (GnomeCanvasItem *item,
 			     GnomeCanvasBuf *buf)
 {
 	GnomeCanvasBpath *bpath;
-	guint32 fg_color, bg_color;
 
 	bpath = GNOME_CANVAS_BPATH (item);
 
@@ -640,73 +653,72 @@ gnome_canvas_bpath_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_p
 	if (parent_class->update)
 		(* parent_class->update) (item, affine, clip_path, flags);
 
-	if (item->canvas->aa) {
-		gnome_canvas_item_reset_bounds (item);
-
-		affine_bpath = art_bpath_affine_transform (bpath->priv->bpath,
-							   affine);
-
-		vpath = art_bez_path_to_vec (affine_bpath, 0.25);
-
-		art_free (affine_bpath);
-
-		if (bpath->fill_set) {
-			if (bpath->wind == GNOME_CANVAS_BPATH_RAW) {
-				svp = art_svp_from_vpath (vpath);
-			} else {
-				ArtVpath *perturbed_vpath;
-				ArtSVP *tmp_svp;
-				ArtWindRule art_wind;
-
-				perturbed_vpath = art_vpath_perturb (vpath);
-				svp = art_svp_from_vpath (perturbed_vpath);
-				art_free (perturbed_vpath);
-				tmp_svp = art_svp_uncross (svp);
-				art_svp_free (svp);
-				if (bpath->wind == GNOME_CANVAS_BPATH_NONZERO) {
-					art_wind = ART_WIND_RULE_NONZERO;
-				} else {
-					art_wind = ART_WIND_RULE_ODDEVEN;
-				}
-				svp = art_svp_rewind_uncrossed (tmp_svp, art_wind);
-				art_svp_free (tmp_svp);
-			}
-			gnome_canvas_item_update_svp_clip (item, &bpath->fill_svp, svp, clip_path);
-		}
-
-		if (bpath->outline_set) {
-			if (bpath->width_pixels)
-				width = bpath->width;
-			else
-				width = bpath->width * item->canvas->pixels_per_unit;
-		
-			if (width < 0.5)
-				width = 0.5;
-		
-			if (dash)
-			  {
-			    int i;
-			    ArtVpath *old = vpath;
-			    
-			    for (i = 0; i < dash->n_dash; i++)
-			      dash->dash[i] *= item->canvas->pixels_per_unit;
-
-			    vpath = art_vpath_dash (old, dash);
-			    art_free (old);
-			  }
+	gnome_canvas_item_reset_bounds (item);
+	
+	affine_bpath = art_bpath_affine_transform (bpath->priv->bpath,
+						   affine);
+	
+	vpath = art_bez_path_to_vec (affine_bpath, 0.25);
+	
+	art_free (affine_bpath);
+	
+	if (bpath->fill_set) {
+		if (bpath->wind == GNOME_CANVAS_BPATH_RAW) {
+			svp = art_svp_from_vpath (vpath);
+		} else {
+			ArtVpath *perturbed_vpath;
+			ArtSVP *tmp_svp;
+			ArtWindRule art_wind;
 			
-			svp = art_svp_vpath_stroke (vpath,
-						    gnome_canvas_join_gdk_to_art (bpath->join),
-						    gnome_canvas_cap_gdk_to_art (bpath->cap),
-						    width,
-						    4,
-						    0.25);
-
-			gnome_canvas_item_update_svp_clip (item, &bpath->outline_svp, svp, clip_path);
+			perturbed_vpath = art_vpath_perturb (vpath);
+			svp = art_svp_from_vpath (perturbed_vpath);
+			art_free (perturbed_vpath);
+			tmp_svp = art_svp_uncross (svp);
+			art_svp_free (svp);
+			if (bpath->wind == GNOME_CANVAS_BPATH_NONZERO) {
+				art_wind = ART_WIND_RULE_NONZERO;
+			} else {
+				art_wind = ART_WIND_RULE_ODDEVEN;
+			}
+			svp = art_svp_rewind_uncrossed (tmp_svp, art_wind);
+			art_svp_free (tmp_svp);
 		}
-		art_free (vpath);
-
-	} else {
+		gnome_canvas_item_update_svp_clip (item, &bpath->fill_svp, svp, clip_path);
+	}
+	
+	if (bpath->outline_set) {
+		if (bpath->width_pixels)
+			width = bpath->width;
+		else
+			width = bpath->width * item->canvas->pixels_per_unit;
+		
+		if (width < 0.5)
+			width = 0.5;
+		
+		if (dash)
+		{
+			int i;
+			ArtVpath *old = vpath;
+			
+			for (i = 0; i < dash->n_dash; i++)
+				dash->dash[i] *= item->canvas->pixels_per_unit;
+			
+			vpath = art_vpath_dash (old, dash);
+			art_free (old);
+		}
+		
+		svp = art_svp_vpath_stroke (vpath,
+					    gnome_canvas_join_gdk_to_art (bpath->join),
+					    gnome_canvas_cap_gdk_to_art (bpath->cap),
+					    width,
+					    4,
+					    0.25);
+		
+		gnome_canvas_item_update_svp_clip (item, &bpath->outline_svp, svp, clip_path);
+	}
+	art_free (vpath);
+	
+	if (!item->canvas->aa){
 		set_outline_gc_width (bpath);
 		set_gc_foreground (bpath->fill_gc, bpath->fill_pixel);
 		set_gc_foreground (bpath->outline_gc, bpath->outline_pixel);
@@ -746,7 +758,8 @@ gnome_canvas_bpath_unrealize (GnomeCanvasItem *item)
 		(* parent_class->unrealize) (item);
 }
 
-/* Converts an array of world coordinates into an array of canvas pixel coordinates.  Takes in the
+/*
+ * Converts an array of world coordinates into an array of canvas pixel coordinates.  Takes in the
  * item->world deltas and the drawable deltas.
  */
 static void
@@ -774,58 +787,62 @@ item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coord
 	}
 }
 
+static GdkPoint *
+vpath_to_points (ArtVpath *path, int *n)
+{
+	ArtVpath *p = path;
+	GdkPoint *points;
+	int i;
+	
+	*n = 0;
+	for (p = path; p->code != ART_END; p++)
+		(*n)++;
+
+	points = g_new (GdkPoint, *n);
+
+	for (i = 0, p = path; p->code != ART_END; p++, i++){
+		points [i].x = p->x;
+		points [i].y = p->y;
+	}
+
+	return points;
+}
+
 static void
 gnome_canvas_bpath_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			   int x, int y, int width, int height)
 {
-#if 0
-	/* TODO: implement this */
-	GnomeCanvasBpath *bpath;
-	GdkPoint static_points[NUM_STATIC_POINTS];
+	GnomeCanvasBpath *bpath, *affine_bpath;
+	ArtVpath *vpath;
+	double i2c [6];
+	int n;
 	GdkPoint *points;
-	double dx, dy;
-	int cx, cy;
-	int i;
-	double i2c[6];
 
 	bpath = GNOME_CANVAS_BPATH (item);
 
-	if (bpath->num_points == 0)
-		return;
-
-	/* Build array of canvas pixel coordinates */
-
-	if (bpath->num_points <= NUM_STATIC_POINTS)
-		points = static_points;
-	else
-		points = g_new (GdkPoint, bpath->num_points);
-
 	gnome_canvas_item_i2c_affine (item, i2c);
-
 	i2c[4] -= x;
 	i2c[5] -= y;
-
-	item_to_canvas (item->canvas, bpath->coords, points, bpath->num_points, i2c);
+	affine_bpath = art_bpath_affine_transform (bpath->priv->bpath, i2c);
+	vpath = art_bez_path_to_vec (affine_bpath, 0.25);
+	art_free (affine_bpath);
+	points = vpath_to_points (vpath, &n);
 
 	if (bpath->fill_set) {
 		if (bpath->fill_stipple)
 			gnome_canvas_set_stipple_origin (item->canvas, bpath->fill_gc);
 
-		gdk_draw_bpath (drawable, bpath->fill_gc, TRUE, points, bpath->num_points);
+		gdk_draw_polygon (drawable, bpath->fill_gc, TRUE, points, n);
 	}
 
 	if (bpath->outline_set) {
 		if (bpath->outline_stipple)
 			gnome_canvas_set_stipple_origin (item->canvas, bpath->outline_gc);
 
-		gdk_draw_bpath (drawable, bpath->outline_gc, FALSE, points, bpath->num_points);
+		gdk_draw_polygon (drawable, bpath->outline_gc, FALSE, points, n);
 	}
 
-	/* Done */
-
-	if (points != static_points)
-		g_free (points);
-#endif
+	g_free (points);
 }
 
 static double
