@@ -581,10 +581,11 @@ gnome_font_refresh_fontmap (GnomeFontClass *class)
     {
       fontmap_fn = g_malloc (strlen (env_home) + 32);
       sprintf (fontmap_fn, "%s/.gnome/fonts/fontmap", env_home);
-      gnome_font_load_fontmap (class, fontmap_fn);
+      if (g_file_exists (fontmap_fn))
+	      gnome_font_load_fontmap (class, fontmap_fn);
       g_free (fontmap_fn);
       fontmap_fn = gnome_datadir_file ("fonts/fontmap");
-      if (fontmap_fn)
+      if (fontmap_fn && g_file_exists (fontmap_fn))
 	{
 	  gnome_font_load_fontmap (class, fontmap_fn);
 	  g_free (fontmap_fn);
@@ -593,7 +594,8 @@ gnome_font_refresh_fontmap (GnomeFontClass *class)
         {
           /* Check gnome-print installation prefix */
           fontmap_fn = g_strconcat(DATADIR, "/fonts/fontmap", NULL);
-          gnome_font_load_fontmap (class, fontmap_fn);
+	  if (g_file_exists (fontmap_fn))
+	      gnome_font_load_fontmap (class, fontmap_fn);
           g_free (fontmap_fn);
         }
     }
