@@ -435,6 +435,7 @@ gnome_print_meta_init (GnomePrintMeta *meta)
 
 	strcpy (meta->buffer, GNOME_METAFILE_SIGNATURE);
 	meta->current = sizeof (GnomeMetaFileHeader);
+	((GnomeMetaFileHeader *)meta->buffer)->size = -1;
 }
 
 /**
@@ -852,6 +853,10 @@ gnome_print_meta_render (GnomePrintContext *destination, void *meta_stream)
 	if (strcmp ((char *) meta_stream, GNOME_METAFILE_SIGNATURE) != 0)
 		return FALSE;
 
+	if (((GnomeMetaFileHeader *)meta_stream)->size == -1){
+		g_warning ("This stream was not closed");
+		return FALSE;
+	}
 	data = meta_stream;
 	size = ((GnomeMetaFileHeader *)meta_stream)->size;
 	return do_render (destination, data + sizeof (GnomeMetaFileHeader), size, -1);
@@ -904,6 +909,11 @@ gnome_print_meta_render_page (GnomePrintContext *destination, void *meta_stream,
 
 	data = meta_stream;
 	size = ((GnomeMetaFileHeader *)meta_stream)->size;
+
+	if (size == -1){
+		g_warning ("This printing context has not been closed");
+		return FALSE;
+	}
 	return do_render (destination, data + sizeof (GnomeMetaFileHeader), size, page);
 }
 
