@@ -47,6 +47,7 @@ typedef struct {
 	double       affine [6];
 	double       page_affine [6];
 	double       base_affine [6];
+	double       startup_page_affine [6];
 } GraphicContext;
 
 struct _GnomePrintPreviewPrivate {
@@ -459,7 +460,7 @@ gpp_concat (GnomePrintContext *pc, double matrix [6])
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	art_affine_multiply (gc (priv)->affine, matrix, gc (priv)->affine);
+	art_affine_multiply (gc (priv)->affine,  matrix, gc (priv)->affine);
 
 	return 1;
 }
@@ -520,6 +521,16 @@ gpp_gsave (GnomePrintContext *pc)
 	return 1;
 }
 
+static void
+pop_state (GnomePrintContext *pc)
+{
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+	GnomePrintPreviewPrivate *priv = pp->priv;
+
+	gc_free (gc (priv));
+	priv->context_list = g_slist_remove (priv->context_list, gc (priv));
+}
+
 static int
 gpp_grestore (GnomePrintContext *pc)
 {
@@ -530,10 +541,8 @@ gpp_grestore (GnomePrintContext *pc)
 		g_warning ("grestore invoked on emtpy stack");
 		return -1;
 	}
-	
-	gc_free (gc (priv));
-	priv->context_list = g_slist_remove (priv->context_list, gc (priv));
 
+	pop_state (pc);
 	return 1;
 }
 
@@ -624,7 +633,7 @@ gpp_image (GnomePrintContext *pc, char *data, int width, int height, int rowstri
 		art_affine_multiply (transform, flip, g->affine);
 		art_affine_multiply (transform, transform, g->page_affine);
 		
-		gnome_canvas_item_affine_absolute (item, transform);
+		gnome_canvas_item_affine_relative (item, transform);
 	}
 	
 	return 1;
@@ -668,6 +677,15 @@ gpp_showpage (GnomePrintContext *pc)
 		"y", 0.0,
 		NULL);
 
+	/*
+	 * Reset affine and path
+	 */
+	art_affine_identity (gc (priv)->affine);
+
+	/*
+	 * Kill current path
+	 */
+	gc_clear_bpath (gc (priv));
 	return 0;
 }
 
@@ -705,9 +723,10 @@ gpp_show (GnomePrintContext *pc, char const *text)
 	GnomeCanvasItem *item;
 	double x, y;
 	Gt1LoadedFont *gt1_font;
-	double transform [6];
+	double transform [6], inverse[6];
 	GraphicContext *g = gc (priv);
 
+
 	if (g->font == NULL){
 		g_warning ("Show invoked without prior setfont");
 		return -1;
@@ -735,12 +754,28 @@ gpp_show (GnomePrintContext *pc, char const *text)
 	x = g->bpath->bpath [0].x3;
 	y = g->bpath->bpath [0].y3;
 
+	/*
+	 * Flip the angle
+	 */
+	memcpy (inverse, g->affine, sizeof (g->affine));
+       
+	inverse [1] = -inverse [1];
+        inverse [2] = -inverse [2];
+       
+	/*
+	inverse [0] *= pp->canvas->pixels_per_unit;
+        inverse [3] *= pp->canvas->pixels_per_unit;
+	*/
+	art_affine_invert (transform, inverse);
+	x =  g->bpath->bpath [0].x3 * transform[0] + g->bpath->bpath [0].y3  * transform[2] + transform[4];
+	y =  g->bpath->bpath [0].x3 * transform[1] + g->bpath->bpath [0].y3  * transform[3] + transform[5]; 
+
 	lazy_showpage_check (priv);
 	item = gnome_canvas_item_new (
 		GNOME_CANVAS_GROUP (priv->page),
 		gnome_canvas_hacktext_get_type (),
-		"x",           x * pp->canvas->pixels_per_unit,
-		"y",           y * pp->canvas->pixels_per_unit,
+		"x",           x,
+		"y",           y,
 		"text",        text,
 		"font",        gt1_font,
 		"size",        g->font->size,
@@ -751,10 +786,11 @@ gpp_show (GnomePrintContext *pc, char const *text)
 	 * Flip the angle
 	 */
 	memcpy (transform, g->affine, sizeof (g->affine));
+       
 	transform [1] = -transform [1];
 	transform [2] = -transform [2];
+      
 	gnome_canvas_item_affine_absolute (item, transform);
-
 	priv->current_page++;
 	
 	gc_clear_bpath (gc (priv));
@@ -771,8 +807,7 @@ static gboolean
 kill_font (gpointer key, gpointer value, gpointer user_data)
 {
 	g_free (key);
-
-	/* FIXME: free font */
+	gt1_unload_font (value);
 	return TRUE;
 }
 
@@ -786,6 +821,7 @@ gpp_finalize (GtkObject *object)
 
 	for (l = priv->context_list; l; l = l->next)
 		gc_free (l->data);
+	g_slist_free (priv->context_list);
 	
 	if (pp->canvas)
 		gtk_object_unref (GTK_OBJECT (pp->canvas));
