@@ -169,11 +169,13 @@ gnome_print_ps_setlinecap (GnomePrintContext *pc, int captype)
 static int
 gnome_print_ps_setdash (GnomePrintContext *pc, int n_values, double *values, double offset)
 {
-  return gnome_print_context_fprintf (pc,
-                                      "%d %g %g setdash\n",
-                                      n_values,
-                                      values,
-                                      offset);
+  int i, res;
+
+  res = gnome_print_context_fprintf(pc, "[");
+  for (i = 0; i < n_values; i++)
+    res += gnome_print_context_fprintf(pc, " %g", values[i]);
+  res += gnome_print_context_fprintf(pc, "] %g setdash\n", offset);
+  return res;
 }
 
 static int
