@@ -102,7 +102,9 @@ struct _GnomePrintContextClass
 
 
 GtkType            gnome_print_context_get_type (void);
-GnomePrintContext *gnome_print_context_new      (GnomePrinter *printer, const char *paper_size);
+GnomePrintContext *gnome_print_context_new      (GnomePrinter *printer);
+GnomePrintContext *gnome_print_context_new_with_paper_size
+                               (GnomePrinter *, const char *paper_size);
 int                gnome_print_context_close    (GnomePrintContext *pc);
 
 
