@@ -49,7 +49,7 @@ set_profile (GnomePrinterWidget *gpw, GnomePrinterProfile *pp)
 	printer = gnome_printer_widget_get_printer (gpw);
 	msg = gnome_printer_str_status (gnome_printer_get_status (printer));
 	gtk_label_set_text (GTK_LABEL (gpw->label_state), msg);
-	gtk_object_destroy (GTK_OBJECT (printer));
+	gtk_object_unref (GTK_OBJECT (printer));
 
 	/*
 	 * Load the rest of the information fields
@@ -298,9 +298,7 @@ gnome_printer_widget_get_printer (GnomePrinterWidget *widget)
 		char *com = gtk_entry_get_text (GTK_ENTRY (gpw->entry_command));
 
 		printer = gnome_printer_profile_get_printer (gpw->profile, NULL, com);
-	}
-
-	if (GTK_TOGGLE_BUTTON (gpw->r2)->active){
+	} else if (GTK_TOGGLE_BUTTON (gpw->r2)->active){
 		char *fn = gtk_entry_get_text (GTK_ENTRY (gpw->entry_filename));
 		
 		printer = gnome_printer_profile_get_printer (gpw->profile, fn, NULL);
