@@ -259,12 +259,12 @@ gnome_printer_profile_get_output (GnomePrinterProfile *pp)
 }
 
 static GnomePrinter *
-gnome_printer_create (char *file, int type)
+gnome_printer_create (char *file, char *driver)
 {
 	GnomePrinter *printer;
 	
 	printer = gtk_type_new (gnome_printer_get_type ());
-	printer->printer_type = type;
+	printer->driver = g_strdup (driver);
 	printer->filename = g_strdup (file);
 	return printer;
 }
@@ -315,8 +315,7 @@ gnome_printer_profile_get_printer (GnomePrinterProfile *pp, char *optional_file,
 	if (!output)
 		output = g_strdup ("gnome-printer-output");
 	
-	if (strcmp (pp->driver, "gnome-print-ps") == 0)
-		printer = gnome_printer_create (output, GNOME_PRINTER_TYPE_GENERIC_PS);
+	printer = gnome_printer_create (output, pp->driver);
 
 	g_free (output);
 	
