@@ -65,7 +65,7 @@ gnome_printer_new_generic_ps (const char *filename)
   GnomePrinter *printer;
 
   printer = gtk_type_new (gnome_printer_get_type ());
-  printer->printer_type = GNOME_PRINTER_TYPE_GENERIC_PS;
+  printer->driver = g_strdup ("gnome-print-ps");
   printer->filename = g_strdup (filename);
 
   return printer;
@@ -84,7 +84,9 @@ gnome_printer_finalize (GtkObject *object)
 
   if (printer->filename)
     g_free (printer->filename);
-
+  if (printer->driver)
+    g_free (printer->driver);
+  
   (* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
