@@ -21,15 +21,11 @@ extern "C" {
 typedef struct _GnomePrinter       GnomePrinter;
 typedef struct _GnomePrinterClass  GnomePrinterClass;
 
-typedef enum {
-  GNOME_PRINTER_TYPE_GENERIC_PS
-} GnomePrinterType;
-
 struct _GnomePrinter
 {
   GtkObject object;
 
-  GnomePrinterType printer_type;
+  char *driver;
   char *filename;
 };
 
