@@ -10,8 +10,8 @@
 */
 
 typedef struct _Gt1LoadedFont Gt1LoadedFont;
-Gt1LoadedFont *
-gt1_load_font (char *filename);
+Gt1LoadedFont * gt1_load_font (const char *filename);
+void gt1_unload_font (Gt1LoadedFont *);
 
 ArtBpath *
 gt1_get_glyph_outline (Gt1LoadedFont *font, int glyphnum, double *p_wx);
