@@ -1,4 +1,4 @@
-/* My mods:
+/* Our mods:
 
 1. FontInfo has become Font_Info to avoid namespace collisions.
 2. Version is a linetoken because Bitstream Charter has a space in the version.
@@ -8,6 +8,13 @@
 6. Stopped buffer overflows in token and linetoken.
 
 Raph Levien <raph@acm.org> writing on 4 Oct 1998, updating 21 Oct 1998
+
+
+1. parseFileFree function.
+2. Leak fix in parseFile.
+
+Morten Welinder <terra@diku.dk> September 1999.
+
 */
 
 /*
@@ -1101,7 +1108,8 @@ extern int parseFile (fp, fi, flags)
     
    			      
     /* storage data for the global variable ident */			      
-    ident = (char *) calloc(MAX_NAME, sizeof(char)); 
+    if (!ident)
+	 ident = (char *) calloc(MAX_NAME, sizeof(char)); 
     if (ident == NULL) {error = AFM_storageProblem; return(error);}      
   
     (*fi) = (Font_Info *) calloc(1, sizeof(Font_Info));
@@ -1242,3 +1250,65 @@ extern int parseFile (fp, fi, flags)
     return(error);
   
 } /* parseFile */
+
+
+void
+parseFileFree (Font_Info *fi)
+{
+     if (fi->gfi) {
+	  free (fi->gfi->afmVersion);
+	  free (fi->gfi->fontName);
+	  free (fi->gfi->fullName);
+	  free (fi->gfi->familyName);
+	  free (fi->gfi->weight);
+	  free (fi->gfi->version);
+	  free (fi->gfi->notice);
+	  free (fi->gfi->encodingScheme);
+	  free (fi->gfi);
+     }
+
+     /* This contains just scalars.  */
+     free (fi->cwi);
+
+     if (fi->cmi) {
+	  int i;
+	  for (i = 0; i < fi->numOfChars; i++) {
+	       free (fi->cmi[i].name);
+	       while (fi->cmi[i].ligs) {
+		    Ligature *tmp;
+		    tmp = fi->cmi[i].ligs;
+		    free (tmp->succ);
+		    free (tmp->lig);
+		    free (tmp);
+		    fi->cmi[i].ligs = fi->cmi[i].ligs->next;
+	       }
+	  }
+	  free (fi->cmi);
+     }
+
+     /* This contains just scalars.  */
+     free (fi->tkd);
+
+     if (fi->pkd) {
+	  int i;
+	  for (i = 0; i < fi->numOfPairs; i++) {
+	       free (fi->pkd[i].name1);
+	       free (fi->pkd[i].name2);
+	  }
+	  free (fi->pkd);
+     }
+
+     if (fi->ccd) {
+	  int i, j;
+	  for (i = 0; i < fi->numOfComps; i++) {
+	       free (fi->ccd[i].ccName);
+	       for (j = 0; j < fi->ccd[i].numOfPieces; j++) {
+		    free (fi->ccd[i].pieces[j].pccName);
+	       }
+	       free (fi->ccd[i].pieces);
+	  }
+	  free (fi->ccd);
+     }
+
+     free (fi);
+}
