@@ -242,7 +242,7 @@ test_load_metafile (char *file)
 	g_free (buffer);
 	
 	printer = gnome_printer_dialog_new_modal ();
-	pc = gnome_print_context_new (printer, "US-Letter");
+	pc = gnome_print_context_new_with_paper_size (printer, "US-Letter");
 
 	gnome_print_meta_render_from_object (pc, meta);
 	gtk_object_unref (GTK_OBJECT (meta));
@@ -279,7 +279,7 @@ test_ps (void)
 		if (!printer)
 			return;
 	
-		pc = gnome_print_context_new (printer, "US-Letter");
+		pc = gnome_print_context_new_with_paper_size (printer, "US-Letter");
 	}
 
 	if (test2)
