@@ -60,7 +60,7 @@ print_test_page (GnomePrintContext *pc)
 		gnome_print_stroke (pc);
 	}
 
-	if (0){
+	if (1){
 
 		double aff [6];
 
@@ -92,7 +92,7 @@ print_test_page (GnomePrintContext *pc)
 		gnome_print_setfont (pc, font);
 		gnome_print_moveto (pc, i, i);
 		gnome_print_show (pc, buf);
-/*		star (pc, i, i); */
+		star (pc, i, i); 
 	}
 
 	gnome_print_showpage (pc);
